/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

#include <jni.h>
#include <errno.h>
#include <termios.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include "com_ianburnett_iilc_drivers_crystalfontz_CFPacketSerial.h"


JNIEXPORT
jlong
JNICALL
Java_com_ianburnett_iilc_drivers_crystalfontz_CFPacketSerial__1writeData (
			JNIEnv *   pEnv,
			jobject    jObj,
			jlong      jLongHandle,
			jbyteArray jByteArray)
{
	/* Required for write() */
	int fd;
	jbyte * buf;
	size_t count;

	/* Returned from write() */
	ssize_t numBytesWritten;


	/* Also sets errno */
	int err_num;


	jboolean isCopy = JNI_FALSE;

	/* Cast down to a file desriptor */
	fd = (int) jLongHandle;

	/* Get a copy of the buffer to write */
	buf = (*pEnv)->GetByteArrayElements(pEnv, jByteArray, &isCopy);

	/* Find out the size of the buffer */
	count = (*pEnv)->GetArrayLength(pEnv, jByteArray);

	/* Perform the write */
	numBytesWritten = write(fd, buf, count);
	err_num = errno;

	/* Release the data from the Java byte[] (no changes made) */
	(*pEnv)->ReleaseByteArrayElements(pEnv, jByteArray, buf, JNI_ABORT);

	/* Return the number of bytes which were actually written */
	return numBytesWritten;
}



JNIEXPORT
jbyteArray
JNICALL
Java_com_ianburnett_iilc_drivers_crystalfontz_CFPacketSerial__1readData (
			JNIEnv * pEnv,
			jobject  jObj,
			jlong    jLongHandle,
			jobject  jObjErrno)
{

	/* Required for read() */
	int fd;
    unsigned char buf[1024];
	size_t count = sizeof(buf) / sizeof(unsigned char);

	/* Returned from read */
	ssize_t numBytesRead;  /* 0 == EOF */

	/* Also sets errno */
	int err_num;

	jbyteArray retArray;
	jsize arrayLen;

	/* Cast down to a file descriptor */
	fd = (int) jLongHandle;

	/* Perform the read */
	numBytesRead = read(fd, buf, count);
	err_num = errno;

	/* Set the errno accordingly */
	if (numBytesRead < 0) {

		/* Create an empty byte array */
		retArray = (*pEnv)->NewByteArray(pEnv, 0);
	}
	else {

		/* Create a new byte array */
		arrayLen = (jsize) numBytesRead;
		retArray = (*pEnv)->NewByteArray(pEnv, arrayLen);

		/* Set the elements */
		(*pEnv)->SetByteArrayRegion(pEnv, retArray, 0, arrayLen, buf);
	}

	return retArray;
}



JNIEXPORT
jlong
JNICALL
Java_com_ianburnett_iilc_drivers_crystalfontz_CFPacketSerial__1initialiseDevice (
			JNIEnv * pEnv,
			jobject  jObj,
			jstring  jStrPort)
{
	jint jnDevice = -1;
	int i;
    struct termios portset;
	char * pszPort = NULL;
	const jchar * jchPort;
	jsize strSz;
	jboolean isCopy = JNI_FALSE;


	/* Retrieve the length of the port string */
	strSz = (*pEnv)->GetStringLength(pEnv, jStrPort);

	/* Allocate enough memory */
	pszPort = malloc(strSz + 1);
	memset(pszPort, 0, strSz+1);

	/* Get the chars */
	jchPort = (*pEnv)->GetStringChars(pEnv, jStrPort, &isCopy);

	/* Copy into the buffer, checking this is all ASCII */
	for (i = 0; i < strSz; i++) {
		jchar c = jchPort[i];
		if (c < 128) {
			pszPort[i] = c;
		}
		else {
			printf("Non-ASCII char found!!\n");
			goto end_of_function;
		}
	}

	/* Release the Java chars */
	(*pEnv)->ReleaseStringChars(pEnv, jStrPort, jchPort);

	/* Open the actual port */
    jnDevice = open(pszPort, O_RDWR | O_NOCTTY | O_NDELAY);
    if (jnDevice == -1) {
		printf("Could not open port [%s]: %s (%d)\n", pszPort, strerror(errno), errno);
		goto end_of_function;
    }

    if (tcgetattr(jnDevice, &portset) == -1) {
        printf("tcgetattr() failed: %s", strerror(errno));
        goto end_of_function;
    }

    cfmakeraw(&portset);
    portset.c_cflag |= 0;
    cfsetispeed(&portset, B115200);
    cfsetospeed(&portset, B115200);
    if (tcsetattr(jnDevice, TCSANOW, &portset) == -1) {
        printf("tcsetattr() failed: %s\n", strerror(errno));
        goto end_of_function;
    }

end_of_function:

	if (pszPort) free(pszPort);
    return jnDevice;
}


